from enigma import eConsoleAppContainer, eDVBDB, eServiceCenter, eServiceReference, iServiceInformation
from Components.Label import Label
from Plugins.Plugin import PluginDescriptor
from Tools.BoundFunction import boundFunction
from ServiceReference import ServiceReference
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Screens.MessageBox import MessageBox
import os
import binascii
from array import array
from string import hexdigits
from datetime import datetime

def findSoftCamKey():
	paths = ["/etc/tuxbox/config/oscam-emu", "/etc/tuxbox/config/oscam", "/etc/tuxbox/config/ncam", "/etc/tuxbox/config", "/etc", "/var/keys", "/usr/keys"]
	if os.path.exists("/tmp/.oscam/oscam.version"):
		data = open("/tmp/.oscam/oscam.version", "r").readlines()
	if os.path.exists("/tmp/.ncam/ncam.version"):
		data = open("/tmp/.ncam/ncam.version", "r").readlines()
		for line in data:
			if "configdir:" in line.lower():
				paths.insert(0, line.split(":")[1].strip())
	for path in paths:
		softcamkey = os.path.join(path, "SoftCam.Key")
		print "[key] the %s exists %d" % (softcamkey, os.path.exists(softcamkey))
		if os.path.exists(softcamkey):
			return softcamkey
	return "/usr/keys/SoftCam.Key"

class HexKeyBoard(VirtualKeyBoard):
	def __init__(self, session, title="", **kwargs):
		VirtualKeyBoard.__init__(self, session, title, **kwargs)
		self.skinName = "VirtualKeyBoard"
		self.keys_list = [[[u"EXIT", u"1", u"2", u"3", u"4", u"5", u"6", u"7", u"8", u"9", u"0", u"BACKSPACE"],
					[u"OK", u"A", u"B", u"C", u"D", u"E", u"F", u"OK", u"LEFT", u"RIGHT", u"ALL", u"CLEAR"]]]
		self.locales = { "hex" : [_("HEX"), _("HEX"), self.keys_list] }
		self.lang = "hex"
		try:
		     self.setLocale()
		except:
		     self.setLang()
		self.buildVirtualKeyBoard()

table = array('L')
for byte in range(256):
	crc = 0
	for bit in range(8):
		if (byte ^ crc) & 1:
			crc = (crc >> 1) ^ 0xEDB88320
		else:
			crc >>= 1
		byte >>= 1
	table.append(crc)

def crc32(string):
	value = 0x2600 ^ 0xffffffffL
	for ch in string:
		value = table[(ord(ch) ^ value) & 0xff] ^ (value >> 8)
	return value ^ 0xffffffffL

def crc323(string):
	value = 0xe00 ^ 0xffffffffL
	for ch in string:
		value = table[(ord(ch) ^ value) & 0xff] ^ (value >> 8)
	return value ^ 0xffffffffL

def hasCAID(session):
	service = session.nav.getCurrentService()
	info = service and service.info()
	caids = info and info.getInfoObject(iServiceInformation.sCAIDs)
	if caids and 0xe00 in caids: return True
	if caids and 0x2600 in caids: return True
	if caids and 0x604 in caids: return True
	try:
		return eDVBDB.getInstance().hasCAID(ref, 0xe00)	
	except:
		pass
	try:
		return eDVBDB.getInstance().hasCAID(ref, 0x2600)	
	except:
		pass
	try:
		return eDVBDB.getInstance().hasCAID(ref, 0x604)	
	except:
		pass
	return False

def getCAIDS(session):
	service = session.nav.getCurrentService()
	info = service and service.info()
	caids = info and info.getInfoObject(iServiceInformation.sCAIDs)
	caidstr = "None"
	if caids: caidstr = " ".join(["%04X (%d)" % (x,x) for x in sorted(caids)])
	return caidstr

def keymenu(session, service=None, *args, **kwargs):
	service = session.nav.getCurrentService()
	info = service and service.info()
	caids = info and info.getInfoObject(iServiceInformation.sCAIDs)
	SoftCamKey = findSoftCamKey()
	ref = session.nav.getCurrentlyPlayingServiceReference()
	if not os.path.exists(SoftCamKey):
		session.open(MessageBox, _("Emu misses SoftCam.Key (%s)" % SoftCamKey), MessageBox.TYPE_ERROR)
	elif not hasCAID(session):
		session.open(MessageBox, _("CAID is missing for service (%s) CAIDS: %s" % (ref.toString(), getCAIDS(session))), MessageBox.TYPE_ERROR)
	else:
	     if caids and 0xe00 in caids:
		   session.openWithCallback(boundFunction(setBISSKeyCallback, session, SoftCamKey), HexKeyBoard,
			title=_("Please enter new key:"), text=findKeyPowerVU(session, SoftCamKey))
	     elif caids and 0x2600 in caids:
		   session.openWithCallback(boundFunction(setBISSKeyCallback, session, SoftCamKey), HexKeyBoard,
			title=_("Please enter new key:"), text=findKeyBISS(session, SoftCamKey))
	     elif caids and 0x604 in caids:
		   session.openWithCallback(boundFunction(setBISSKeyCallback, session, SoftCamKey), HexKeyBoard,
			title=_("Please enter new key:"), text=findKeyIRDETO(session, SoftCamKey))

def setBISSKeyCallback(session, SoftCamKey, key):
	if key: key = "".join(c for c in key if c in hexdigits).upper()
	if key and len(key) == 14:
		if key != findKeyPowerVU(session, SoftCamKey, ""): # no change was made
			keystr = "P %08X 00 %s" % (getHash2(session), key)
			name = ServiceReference(session.nav.getCurrentlyPlayingServiceReference()).getServiceName()
			datastr = "\n%s ; Added on %s for %s at %s\n" % (keystr, datetime.now(), name, getOrb(session))
			open(SoftCamKey, "a").write(datastr)
			eConsoleAppContainer().execute("/etc/init.d/softcam restart")
			session.open(MessageBox, _("PowerVU key saved sucessfuly!%s" % datastr), MessageBox.TYPE_INFO, timeout=10)
	elif key and len(key) == 16:
		if key != findKeyBISS(session, SoftCamKey, ""): # no change was made
			keystr = "F %08X 00 %s" % (getHash(session), key)
			name = ServiceReference(session.nav.getCurrentlyPlayingServiceReference()).getServiceName()
			datastr = "\n%s ; Added on %s for %s at %s\n" % (keystr, datetime.now(), name, getOrb(session))
			open(SoftCamKey, "a").write(datastr)
			eConsoleAppContainer().execute("/etc/init.d/softcam restart")
			session.open(MessageBox, _("BISS key saved sucessfuly!%s" % datastr), MessageBox.TYPE_INFO, timeout=10)
	elif key and len(key) == 32:
		if key != findKeyIRDETO(session, SoftCamKey, ""): # no change was made
			keystr = "I 604 00 %s" % key
			name = ServiceReference(session.nav.getCurrentlyPlayingServiceReference()).getServiceName()
			datastr = "\n%s ; Added on %s for %s at %s\n" % (keystr, datetime.now(), name, getOrb(session))
			open(SoftCamKey, "a").write(datastr)
			eConsoleAppContainer().execute("/etc/init.d/softcam restart")
			session.open(MessageBox, _("IRDETO key saved sucessfuly!%s" % datastr), MessageBox.TYPE_INFO, timeout=10)
	elif key:
		   session.openWithCallback(boundFunction(setBISSKeyCallback, session,SoftCamKey), HexKeyBoard,
			title=_("Invalid key, length is %d" % len(key)), text=key.ljust(16,'*'))
			#title=_("Invalid key, length is %d expecting 16." % len(key)), text=key.ljust(16,'*'))

def getHash(session):
	ref = session.nav.getCurrentlyPlayingServiceReference()
	sid = ref.getUnsignedData(1)
	tsid = ref.getUnsignedData(2)
	onid = ref.getUnsignedData(3)
	namespace = ref.getUnsignedData(4) | 0xA0000000

	# check if we have stripped or full namespace
	if namespace & 0xFFFF == 0:
		# Namespace without frequency - Calculate hash with srvid, tsid, onid and namespace
		data = "%04X%04X%04X%08X" % (sid, tsid, onid, namespace)
	else:
		# Full namespace - Calculate hash with srvid and namespace only
		data = "%04X%08X" % (sid, namespace)
	return crc32(binascii.unhexlify(data))

def getHash2(session):
	ref = session.nav.getCurrentlyPlayingServiceReference()
	sid = ref.getUnsignedData(1)
	tsid = ref.getUnsignedData(2)
	onid = ref.getUnsignedData(3)
	namespace = ref.getUnsignedData(4) | 0xA0000000

	# check if we have stripped or full namespace
	if namespace & 0xFFFF == 0:
		# Namespace without frequency - Calculate hash with srvid, tsid, onid and namespace
		data = "%04X%04X%04X%08X" % (sid, tsid, onid, namespace)
	else:
		# Full namespace - Calculate hash with srvid and namespace only
		data = "%04X%08X" % (sid, namespace)
	return crc323(binascii.unhexlify(data))

def getOrb(session):
	ref = session.nav.getCurrentlyPlayingServiceReference()
	orbpos = ref.getUnsignedData(4) >> 16
	if orbpos == 0xFFFF:
		desc = "C"
	elif orbpos == 0xEEEE:
		desc = "T"
	else:
		if orbpos > 1800: # west
			orbpos = 3600 - orbpos
			h = "W"
		else:
			h = "E"
		desc = ("%d.%d%s") % (orbpos / 10, orbpos % 10, h)
	return desc

def findKeyPowerVU(session, SoftCamKey, key="00000000000000"):
	keystart = "P %08X" % getHash(session)
	keyline = ""
	with open(SoftCamKey, 'rU') as f:
		for line in f:
			if line.startswith(keystart):
				keyline = line
	if keyline:
		return keyline.split()[3]
	else:
		return key

def findKeyBISS(session, SoftCamKey, key="0000000000000000"):
	keystart = "F %08X" % getHash(session)
	keyline = ""
	with open(SoftCamKey, 'rU') as f:
		for line in f:
			if line.startswith(keystart):
				keyline = line
	if keyline:
		return keyline.split()[3]
	else:
		return key

def findKeyIRDETO(session, SoftCamKey, key="00000000000000000000000000000000"):
	keystart = "I 604"
	keyline = ""
	with open(SoftCamKey, 'rU') as f:
		for line in f:
			if line.startswith(keystart):
				keyline = line
	if keyline:
		return keyline.split()[3]
	else:
		return key

def Plugins(**kwargs):
	return [PluginDescriptor(name = "Biscotto" , description = "Manually add Key to current service", icon="plugin.png",
		where = [PluginDescriptor.WHERE_EXTENSIONSMENU, PluginDescriptor.WHERE_PLUGINMENU],
		fnc = keymenu, needsRestart = False)]

